#!/usr/bin/perl -w

use strict;
use utf8;
use Encode;

use HTTP::Request::Common qw(POST GET);
use LWP::UserAgent;

# -----------------------------------------------------------------------------

my %profile = getProfile('./settings.txt');

my $ua = LWP::UserAgent->new;
$ua->agent('Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0;');
$ua->timeout(20);
$ua->credentials('twitter.com:80', 'Twitter API', $profile{id}, $profile{pw});

my $url;
my $response;
my $xmlData;

my $wait = $profile{paging_wait} ? $profile{paging_wait} : 30;

# -----------------------------------------------------------------------------

my %api = (
	'archive' => {
		'url' => 'http://twitter.com/statuses/user_timeline.xml?page=',
		'e' => 'status',
	},
	'favorites' => {
		'url' => 'http://twitter.com/favorites.xml?page=',
		'e' => 'status',
	},
	'replies' => {
		'url' => 'http://twitter.com/replies.xml?page=',
		'e' => 'status',
	},
	'messages' => {
		'url' => 'http://twitter.com/direct_messages.xml?page=',
		'e' => 'direct_message',
	},
	'sent' => {
		'url' => 'http://twitter.com/direct_messages/sent.xml?page=',
		'e' => 'direct_message',
	},
	'following' => {
		'url' => 'http://twitter.com/statuses/friends.xml?lite=true&page=',
		'e' => 'user',
	},
	'followers' => {
		'url' => 'http://twitter.com/statuses/followers.xml?lite=true&page=',
		'e' => 'user',
	},
);



foreach my $key (keys %api) {

	my $page = 1;

	record('>'.$profile{filepath}.$key.'.xml', <<"_EoM_"
<?xml version="1.0" encoding="UTF-8"?>
<?xml-stylesheet type="text/xsl" href="t_memorial.xsl"?>
<$key type="array">

_EoM_
	);


	while (1) {
		$url = $api{$key}{url} . $page;
print "\n-------------\n".$url."\n-------------\n";

		$response = $ua->request(GET($url));

		if ($response->is_success) {
			$xmlData = $response->content;

			if (-1 < index($xmlData, "<$api{$key}{e}>")) {
				record(
					'>>'.$profile{filepath}.$key.'.xml',
					encode('utf8', carve($xmlData, $key))
				);

				$page++;
			} else {
				last;
			}

		} else {
			print "Error: " . $response->status_line . "\n";
			print "(Wait $wait sec.)\n";
			sleep($wait);
		}

		print "(Wait $wait sec.)\n";
		sleep($wait);
	}


	record('>>'.$profile{filepath}.$key.'.xml', <<"_EoM_"

</$key>
_EoM_
	);

	print "\n--\n$key complete?\n\n";
	print "(Wait $wait sec.)\n";
	sleep($wait);
}


print "\n--\nfinish?\n";
exit 0;

# -----------------------------------------------------------------------------

sub carve {
	my ($xmlData, $key) = @_;

	$xmlData =~ s/&#([0-9]+);/chr($1)/ge;
	$xmlData =~ s/&#[xX]([0-9A-Fa-f]+);/chr(hex $1)/ge;

	$xmlData =~ s/^<\?xml .*\n//;

	$xmlData =~ s/^<statuses .*\n//;
	$xmlData =~ s/.*<\/statuses>\n//;

	$xmlData =~ s/^<users .*\n//;
	$xmlData =~ s/.*<\/users>\n//;

	$xmlData =~ s/^<direct\-messages .*\n//;
	$xmlData =~ s/.*<\/direct\-messages>\n//;

	return $xmlData;
}
# -----------------------------------------------------------------------------

sub record {
	my ($record, $data) = @_;

	print $data;


	open (FH , "$record") or die("Could not open $record.");
		flock(FH, 2);
		seek(FH, 0, 2);

		print FH $data;

		truncate(FH, tell(FH));
	close (FH);

	return 1;
}

# -----------------------------------------------------------------------------

sub getProfile {

	# must items
	my @must = qw(
		id
		pw
		filepath
	);


	my $fileName = shift;
	my %profile;

	open (FH , $fileName) or die("Could not open $fileName.");
	my @temp = <FH>;
	close (FH);

	chomp @temp;

	foreach my $row (@temp) {
		($row =~ /^(\S+)?\s*=\s*(.*)/) and do {
			$profile{$1} = $2;
		};
	}

	# check must items.
	foreach (@must) {
		unless ($profile{$_}) {
			die("$_ does not exist in profile.\n");
		}
	}

	return %profile;
}

